
/*
Copyright 2007-2011 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

const EXPORTED_SYMBOLS = ["YaripPage"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;
const Cu = Components.utils;

Cu.import("resource://yarip/object.jsm");
Cu.import("resource://yarip/list.jsm");

function YaripPage(id, name, elementWhitelist, elementBlacklist, elementAttributeList, elementScriptList, contentWhitelist, contentBlacklist, contentRequestHeaderList, contentResponseHeaderList, contentRedirectList, pageStyleList, pageScriptList, pageRequestHeaderList, pageResponseHeaderList, pageRedirectList, pageExtensionList, pageExtendedByList)
{
    this.id = null;
    this.name = "";
    this.elementWhitelist = elementWhitelist ? elementWhitelist : new YaripElementWhitelist("whitelist");
    this.elementBlacklist = elementBlacklist ? elementBlacklist : new YaripElementBlacklist("blacklist");
    this.elementAttributeList = elementAttributeList ? elementAttributeList : new YaripElementAttributeList("attribute");
    this.elementScriptList = elementScriptList ? elementScriptList : new YaripElementScriptList("script");
    this.contentWhitelist = contentWhitelist ? contentWhitelist : new YaripContentWhitelist("whitelist");
    this.contentBlacklist = contentBlacklist ? contentBlacklist : new YaripContentBlacklist("blacklist");
    this.contentRequestHeaderList = contentRequestHeaderList ? contentRequestHeaderList : new YaripHeaderList("request");
    this.contentResponseHeaderList = contentResponseHeaderList ? contentResponseHeaderList : new YaripHeaderList("response");
    this.contentRedirectList = contentRedirectList ? contentRedirectList : new YaripRedirectList("redirect");
    this.pageStyleList = pageStyleList ? pageStyleList : new YaripPageStyleList("style");
    this.pageScriptList = pageScriptList ? pageScriptList : new YaripPageScriptList("script");
    this.pageRequestHeaderList = pageRequestHeaderList ? pageRequestHeaderList : new YaripHeaderList("request");
    this.pageResponseHeaderList = pageResponseHeaderList ? pageResponseHeaderList : new YaripHeaderList("response");
    this.pageRedirectList = pageRedirectList ? pageRedirectList : new YaripRedirectList("redirect");
    this.pageExtensionList = pageExtensionList ? pageExtensionList : new YaripPageExtensionList("extension");
    this.pageExtendedByList = pageExtendedByList ? pageExtendedByList : new YaripPageExtendedByList();
    this.temporary = false;

    this.setId(id);
    this.setName(name, true);
}
YaripPage.prototype = new YaripObject;
YaripPage.prototype.constructor = YaripPage;
YaripPage.prototype.getKey = function()
{
    return this.getId();
}
YaripPage.prototype.setId = function(value)
{
    if (!value) return;

    this.id = "" + value;
    this.pageExtensionList.setId(this.id);
    this.pageExtendedByList.setId(this.id);
}
YaripPage.prototype.getId = function()
{
    return this.id;
}
YaripPage.prototype.setName = function(value, init)
{
    if (value) {
        this.name = "" + value;
        if (init) this.init();
    }
}
YaripPage.prototype.getName = function()
{
    return this.name;
}
YaripPage.prototype.isEmpty = function()
{
    return this.elementWhitelist.isEmpty()
        && this.elementBlacklist.isEmpty()
        && this.elementAttributeList.isEmpty()
        && this.elementScriptList.isEmpty()
        && this.contentWhitelist.isEmpty()
        && this.contentBlacklist.isEmpty()
        && this.contentRequestHeaderList.isEmpty()
        && this.contentResponseHeaderList.isEmpty()
        && this.contentRedirectList.isEmpty()
        && this.pageStyleList.isEmpty()
        && this.pageScriptList.isEmpty()
        && this.pageRequestHeaderList.isEmpty()
        && this.pageResponseHeaderList.isEmpty()
        && this.pageRedirectList.isEmpty()
        && this.pageExtensionList.isEmpty()
        && this.pageExtendedByList.isEmpty();
}
YaripPage.prototype.hasElements = function()
{
    return !this.elementWhitelist.isEmpty()
        || !this.elementBlacklist.isEmpty()
        || !this.elementAttributeList.isEmpty()
        || !this.pageStyleList.isEmpty();
}
YaripPage.prototype.hasContents = function()
{
    return !this.contentWhitelist.isEmpty()
        || !this.contentBlacklist.isEmpty();
}
YaripPage.prototype.hasScripts = function()
{
    return !this.elementScriptList.isEmpty()
        || !this.pageScriptList.isEmpty();
}
YaripPage.prototype.hasHeaders = function()
{
    return !this.contentRequestHeaderList.isEmpty()
        || !this.contentResponseHeaderList.isEmpty()
        || !this.pageRequestHeaderList.isEmpty()
        || !this.pageResponseHeaderList.isEmpty();
}
YaripPage.prototype.hasRedirects = function()
{
    return !this.contentRedirectList.isEmpty()
        || !this.pageRedirectList.isEmpty();
}
YaripPage.prototype.setTemporary = function(value)
{
    this.temporary = "" + value == "true";
}
YaripPage.prototype.getTemporary = function()
{
    return this.temporary;
}
//YaripPage.prototype.clone = function(purge, pageName)
YaripPage.prototype.clone = function(purge, pageName, id)
{
    var origId = this.id;
    if (purge || id) {
        this.id = id ? id : this.newId();
        this.pageExtensionList.id = this.id;
        this.pageExtendedByList.id = this.id;
    }
    var tmp = new this.constructor(
            this.id,
            pageName ? pageName : this.name,
            this.elementWhitelist.clone(purge),
            this.elementBlacklist.clone(purge),
            this.elementAttributeList.clone(purge),
            this.elementScriptList.clone(purge),
            this.contentWhitelist.clone(purge),
            this.contentBlacklist.clone(purge),
            this.contentRequestHeaderList.clone(purge),
            this.contentResponseHeaderList.clone(purge),
            this.contentRedirectList.clone(purge),
            this.pageStyleList.clone(),
            this.pageScriptList.clone(),
            this.pageRequestHeaderList.clone(purge),
            this.pageResponseHeaderList.clone(purge),
            this.pageRedirectList.clone(purge),
            this.pageExtensionList.clone(purge),
            purge ? null : this.pageExtendedByList.clone());
    if (purge || id) {
        this.id = origId;
        this.pageExtensionList.id = this.id;
        this.pageExtendedByList.id = this.id;
    }
    return tmp;
}
YaripPage.prototype.merge = function(page)
{
    if (!page) return;
    this.elementWhitelist.merge(page.elementWhitelist);
    this.elementBlacklist.merge(page.elementBlacklist);
    this.elementAttributeList.merge(page.elementAttributeList);
    this.elementScriptList.merge(page.elementScriptList);
    this.contentWhitelist.merge(page.contentWhitelist);
    this.contentBlacklist.merge(page.contentBlacklist);
    this.contentRequestHeaderList.merge(page.contentRequestHeaderList);
    this.contentResponseHeaderList.merge(page.contentResponseHeaderList);
    this.contentRedirectList.merge(page.contentRedirectList);
    this.pageStyleList.merge(page.pageStyleList);
    this.pageScriptList.merge(page.pageScriptList);
    this.pageRequestHeaderList.merge(page.pageRequestHeaderList);
    this.pageResponseHeaderList.merge(page.pageResponseHeaderList);
    this.pageRedirectList.merge(page.pageRedirectList);
    this.pageExtensionList.merge(page.pageExtensionList);
    this.pageExtensionList.removeByKey(this.getId());
    this.pageExtendedByList.merge(page.pageExtendedByList);
    this.pageExtendedByList.removeByKey(this.getId());
    this.setTemporary(this.getTemporary() || page.getTemporary());
}
//YaripPage.prototype.subtract = function(page)
//{
//    if (!page) return;
//    this.elementWhitelist.subtract(page.elementWhitelist);
//    this.elementBlacklist.subtract(page.elementBlacklist);
//    this.elementAttributeList.subtract(page.elementAttributeList);
//    this.elementScriptList.subtract(page.elementScriptList);
//    this.contentWhitelist.subtract(page.contentWhitelist);
//    this.contentBlacklist.subtract(page.contentBlacklist);
//    this.contentRequestHeaderList.subtract(page.contentRequestHeaderList);
//    this.contentResponseHeaderList.subtract(page.contentResponseHeaderList);
//    this.contentRedirectList.subtract(page.contentRedirectList);
//    this.pageStyleList.subtract(page.pageStyleList);
//    this.pageScriptList.subtract(page.pageScriptList);
//    this.pageRequestHeaderList.subtract(page.pageRequestHeaderList);
//    this.pageResponseHeaderList.subtract(page.pageResponseHeaderList);
//    this.pageRedirectList.subtract(page.pageRedirectList);
//    this.pageExtensionList.subtract(page.pageExtensionList);
//    this.setTemporary(false);
//}
YaripPage.prototype.purge = function()
{
    this.elementWhitelist.purge();
    this.elementBlacklist.purge();
    this.elementAttributeList.purge();
    this.elementScriptList.purge();
    this.contentWhitelist.purge();
    this.contentBlacklist.purge();
    this.contentRequestHeaderList.purge();
    this.contentResponseHeaderList.purge();
    this.contentRedirectList.purge();
    this.pageRequestHeaderList.purge();
    this.pageResponseHeaderList.purge();
    this.pageRedirectList.purge();
    this.setTemporary(false);
}

const IP = 0;
const SIMPLE = 1;
const DOMAIN = 2;
const UNKNOWN = 3;
YaripPage.prototype.obj = {
    type: UNKNOWN,
    protocol: "",
    ipArr: [],
    tldArr: [],
    domainArr: []
};
YaripPage.prototype.init = function()
{
    if (!this.getName()) return;

    var matches = this.getName().match(this.getRegExp("uriSimple"));
//if (/yarip/.test(this.getName())) {
//    for (var i = 0; i < matches.length; i++) {
//        if (matches[i]) dump("matches["+i+"]="+matches[i]+"\n");
//    }
//}
    if (matches) {
        var protocol = matches[4] || matches[5] || "";
        var ip = matches[14] || matches[99] || "";
//            var domain = matches[12] || matches[172] || matches[184] || matches[191] || matches[677] || "";
//            var domain = matches[12] || matches[172] || matches[184] || matches[191] || matches[212] || matches[677] || "";
        var domain = matches[12] || matches[172] || matches[184] || matches[191] || matches[206] || matches[212] || matches[677] || "";
        if (domain && !this.getRegExp("findTld").test(domain)) {
            var simple = domain;
            domain = "";
        } else {
            var simple = "";
        }
        var type = domain ? DOMAIN : ip ? IP : simple ? SIMPLE : UNKNOWN;
    } else {
        var protocol = "";
        var ip = "";
        var domain = "";
        var simple = "";
        var type = UNKNOWN;
    }

    var ipArr = [];
    var tldArr = [];
    var domainArr = [];
    switch (type) {
    case IP:
        ipArr = ip.split('.');
        break;
    case SIMPLE:
        domainArr = simple.split('.').reverse();
        break;
    case DOMAIN:
        tldArr = domain.match(this.getRegExp("findTld"))[0].split('.');
        domainArr = domain.split('.').reverse();
        break;
    }

    this.obj = {
        type: type,
        protocol: protocol,
        ipArr: ipArr || [],
        tldArr: tldArr || [],
        domainArr: domainArr || []
    }
}
YaripPage.prototype.compare = function(b)
{
    if (!b) return 1;

    // Comparing types
    var aType = this.obj.type;
    var bType = b.obj.type;
    if (aType < bType) return -1;
    if (aType > bType) return 1;

    // Same types
    switch (aType) {
    case IP:
        // Comparing IPs
        var aIpArr = this.obj.ipArr;
        var bIpArr = b.obj.ipArr;
        if (aIpArr < bIpArr) return -1;
        if (aIpArr > bIpArr) return 1;
        break;

    case SIMPLE:
        // Comparing DOMAINs (SIMPLE)
        var aDomainArr = this.obj.domainArr;
        var bDomainArr = b.obj.domainArr;
        if (aDomainArr < bDomainArr) return -1;
        if (aDomainArr > bDomainArr) return 1;
        break;

    case DOMAIN:
        // Comparing TLDs
        var aTldArr = this.obj.tldArr;
        var bTldArr = b.obj.tldArr;
        if (aTldArr < bTldArr) return -1;
        if (aTldArr > bTldArr) return 1;
        // Comparing DOMAINs
        var aDomainArr = this.obj.domainArr;
        var bDomainArr = b.obj.domainArr;
        if (aDomainArr < bDomainArr) return -1;
        if (aDomainArr > bDomainArr) return 1;
        // Comparing PROTOCOLs
        var aProtocol = this.obj.protocol;
        var bProtocol = b.obj.protocol;
        if (aProtocol < bProtocol) return -1;
        if (aProtocol > bProtocol) return 1;
        break;
    }

    var aName = this.getName();
    var bName = b.getName();
    if (aName < bName) return -1;
    if (aName > bName) return 1;
    return 0;
}
YaripPage.prototype.generateXml = function()
{
    if (this.temporary) return "";

    var tmpElement = this.elementWhitelist.generateXml() +
        this.elementBlacklist.generateXml() +
        this.elementAttributeList.generateXml() +
        this.elementScriptList.generateXml();
    if (tmpElement != "") {
        tmpElement = "\t\t<element>\n" + tmpElement + "\t\t</element>\n";
    }

    var tmpContentHeader = this.contentRequestHeaderList.generateXml() +
        this.contentResponseHeaderList.generateXml();
    var tmpContent = this.contentWhitelist.generateXml() +
        this.contentBlacklist.generateXml() +
        (tmpContentHeader != "" ? "\t\t\t<header>\n" + tmpContentHeader + "\t\t\t</header>\n" : "") +
        this.contentRedirectList.generateXml();
    if (tmpContent != "") {
        tmpContent = "\t\t<content>\n" + tmpContent + "\t\t</content>\n";
    }

    var tmpPageHeader = this.pageRequestHeaderList.generateXml() +
        this.pageResponseHeaderList.generateXml();
    var tmpPage = this.pageStyleList.generateXml() +
        this.pageScriptList.generateXml() +
        (tmpPageHeader != "" ? "\t\t\t<header>\n" + tmpPageHeader + "\t\t\t</header>\n" : "") +
        this.pageRedirectList.generateXml() +
        this.pageExtensionList.generateXml();
    if (tmpPage != "") {
        tmpPage = "\t\t<page>\n" + tmpPage + "\t\t</page>\n";
    }

    var tmp = tmpElement + tmpContent + tmpPage;
    if (tmp != "") {
        return "\t<page id=\"" + this.id + "\" name=\"" + this.name + "\">\n" + tmp + "\t</page>\n";
    } else return "";
}
YaripPage.prototype.loadFromObject = function(obj)
{
    this.id = obj.id;
    this.name = obj.name;
    this.elementWhitelist.loadFromObject(obj.elementWhitelist);
    this.elementBlacklist.loadFromObject(obj.elementBlacklist);
    this.elementAttributeList.loadFromObject(obj.elementAttributeList);
    this.elementScriptList.loadFromObject(obj.elementScriptList);
    this.contentWhitelist.loadFromObject(obj.contentWhitelist);
    this.contentBlacklist.loadFromObject(obj.contentBlacklist);
    this.contentRequestHeaderList.loadFromObject(obj.contentRequestHeaderList);
    this.contentResponseHeaderList.loadFromObject(obj.contentResponseHeaderList);
    this.contentRedirectList.loadFromObject(obj.contentRedirectList);
    this.pageStyleList.loadFromObject(obj.pageStyleList);
    this.pageScriptList.loadFromObject(obj.pageScriptList);
    this.pageRequestHeaderList.loadFromObject(obj.pageRequestHeaderList);
    this.pageResponseHeaderList.loadFromObject(obj.pageResponseHeaderList);
    this.pageRedirectList.loadFromObject(obj.pageRedirectList);
    this.pageExtensionList.loadFromObject(obj.pageExtensionList);
    this.pageExtendedByList.loadFromObject(obj.pageExtendedByList);
}

